import boto3
import re
import requests
import json

CLIENT_LAMBDA = boto3.client('lambda')
CLIENT_DB = boto3.client('dynamodb')

#Used to raise exception when required parameters
# are not included in event
class InvalidParameterException(Exception):

    def __init__(self, message):
        super(message)

'''
Use:
   Construct payload to hit Unwired API with info about IoT of M2M device to
   get approx location of device without GPS
'''
def lambda_handler(event, context):
    print("EVENT")
    print(event)

    check_parameters(event)

    payload = contstruct_payload(event)

    response = json.loads(requests.post("https://us1.unwiredlabs.com/v2/process.php", data=json.dumps(payload)).text)

    update_device_with_address(response, event['deviceId'])
    

'''
Parameters: 
  - device_id (deviceId so it can be updated in Foundry)
  - data (returned data from Unwired API call)
Use:
  Construct Payload so device can updated in Foundry via Lambda Invocation
'''
def update_device_with_address(data, device_id):
    payload = {
        'body': {
            'deviceId': device_id
        },
        'invoke': True
    }

    if 'balance' in data:
        payload['body']['a.balance'] = data['balance']
    if 'lat' in data:
        payload['body']['a.lat'] = data['lat']
    if 'lon' in data:
        payload['body']['a.long'] = data['lon']
    if 'address' in data:
        payload['body']['a.address'] = data['address']
    if 'accuracy' in data:
        payload['body']['a.accuracy'] = data['accuracy']
    if 'address_detail' in data:
        if 'county' in data['address_detail']:
            payload['body']['a.county'] = data['address_detail']['county']
        if 'country' in data['address_detail']:
            payload['body']['a.country'] = data['address_detail']['country']
        if 'city' in data['address_detail']:
            payload['body']['a.city'] = data['address_detail']['city']
        if 'state' in data['address_detail']:
            payload['body']['a.state'] = data['address_detail']['state']
        if 'country_code' in data['address_detail']:
            payload['body']['a.country_code'] = data['address_detail']['country_code']
        if 'postal_code' in data['address_detail']:
            payload['body']['a.postal_code'] = data['address_detail']['postal_code']


    CLIENT_LAMBDA.invoke(
        FunctionName='fndy-put-device-secure-dev',
        InvocationType='RequestResponse',
        Payload=json.dumps(payload)
    )

'''
Use:
   Construct payload to include in POST request to Unwired API
'''
def contstruct_payload(event):
    payload = {
        "token": event["token"],
        "radio": event["radio"],
        "mcc": event["mcc"],
        "mnc": event["mnc"],
        "address": 1
    }

    if "address_flag" in event and not re.match('\[.*\]', str(event['address_flag'])):
        payload["address"] = event["address_flag"]
    if "bt" in event and not re.match('\[.*\]', event["bt"]):
        payload["bt"] = event['bt'] 
    if "ip" in event and not re.match('\[.*\]', event["ip"]):
        payload["ip"] = event["ip"] 
    if "cells" in event:
        payload["cells"] = json.loads(event["cells"].replace("'","\""))
    if "wifi" in event:
        payload["wifi"] = json.loads(event["wifi"].replace("'","\""))
    if "fallbacks" in event and not re.match('\[.*\]', event["ip"]):
        payload["fallbacks"] = json.loads(event["fallbacks"].replace("'","\""))
    

    return payload


#Check for valid Parametes
def check_parameters(event):
    if 'token' not in event:
        raise InvalidParameterException('You need an token to invoke this function')
    if 'deviceId' not in event:
        raise InvalidParameterException('You need an deviceId to invoke this function')
    if 'radio' not in event:
        raise InvalidParameterException('You need an radio to invoke this function')
    if 'mcc' not in event:
        raise InvalidParameterException('You need an deviceId to invoke this function')
    if 'radio' not in event:
        raise InvalidParameterException('You need an radio to invoke this function')